/*
Copyright (c) 2008, SFB/TR-55
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef WITCHLIB_H_
#define WITCHLIB_H_

#include "feenlib.h"

#define SPI_REPLY    1
#define SPI_NO_REPLY 0

#define RC_CPLD0_ADDR_REG 0x06
#define RC_CPLD0_DATA_REG 0x05
#define RC_CPLD0_COMM_REG 0x07
#define RC_CPLD0_READ_REG 0x04

#define RC_CPLD1_ADDR_REG 0x0a
#define RC_CPLD1_DATA_REG 0x09
#define RC_CPLD1_COMM_REG 0x0b
#define RC_CPLD1_READ_REG 0x08

#define FPGA_BITSTREAM_OFFSET      0x0
#define FPGA_BITSTREAM_HEADER_SIZE 0x400
#define SLOF_VERSION_OFFSET        0x400038
#define SLOF_VERSION_SIZE          0x10

#define VPD_CASL_OFFSET   0x20
#define VPD_CASL_SIZE     22

#define VPD_OTEMP_OFFSET  0x0C
#define VPD_OTEMP_SIZE    8

#define SP_CTEMP_CMD      0x46
#define SP_CTEMP_SIZE     0x07


/* Standard timeout for rc access */
#define STD_TIMEOUT 90

/*
 * Statistics
 */

void witchStatFeenRestart( feenRC_t );
void witchStatInit( int retry, int mwait );
void witchStatOut( void );

/*
 * Wrappers
 */

ssize_t witchPushFile(feenRC_t rc, const char* filename, int append, const uint8_t *data, size_t length, char **const message );
ssize_t witchPullFile (feenRC_t rc, const char *filename, uint8_t *data, size_t length, char **const message );
int witchServiceCommand( feenRC_t rc, uint8_t addr, uint8_t cmd, const uint8_t *dataIn, uint8_t lengthIn, uint8_t* pidOut, uint8_t* cmdOut, uint8_t* statOut, uint8_t* dataOut, uint8_t* lengthOut, uint8_t* checksumOut, char** const message );

/*
 * RC CPLD0 communication
 */

int witchRCcpld0Reset   ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld0Reset_ep( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld0Read    ( feenRC_t RC, uint8_t* data, char** const message );
int witchRCcpld0Read_ep ( feenRC_t RC, uint8_t* data, char** const message );
int witchRCcpld0Addr    ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld0Addr_ep ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld0Data    ( feenRC_t RC, int data, char** const message );
int witchRCcpld0Data_ep ( feenRC_t RC, int data, char** const message );
int witchRCcpld0Comm    ( feenRC_t RC, uint8_t comm, char** const message );
int witchRCcpld0Comm_ep ( feenRC_t RC, uint8_t comm, char** const message );
int witchRCSpiMux       ( feenRC_t RC, uint8_t Mux, char** const message );

/*
 * RC CPLD1 communication
 */

int witchRCcpld1Reset   ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld1Reset_ep( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld1Read    ( feenRC_t RC, uint8_t* data, char** const message );
int witchRCcpld1Read_ep ( feenRC_t RC, uint8_t* data, char** const message );
int witchRCcpld1Addr    ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld1Addr_ep ( feenRC_t RC, uint8_t addr, char** const message );
int witchRCcpld1Data    ( feenRC_t RC, int data, char** const message );
int witchRCcpld1Data_ep ( feenRC_t RC, int data, char** const message );
int witchRCcpld1Comm    ( feenRC_t RC, uint8_t comm, char** const message );
int witchRCcpld1Comm_ep ( feenRC_t RC, uint8_t comm, char** const message );

/*
 * NC Service Processor communication
 */

int witchNCReadByteCPLD    ( feenRC_t RC, uint8_t addr, uint8_t port, uint8_t* data, char** const message );
int witchNCSpiBackdoorWrite( feenRC_t RC, uint8_t nc, int dcr_addr, int dcr_data, char** const message );
int witchNCSpiBackdoorRead ( feenRC_t RC, uint8_t nc, int dcr_addr, int* const dcr_data, char** const message );
int witchNCvpdRead         ( feenRC_t RC, uint8_t nc, int vpd_addr, uint8_t* vpd_data, uint8_t length, char** const message );
int witchNCvpdWrite        ( feenRC_t RC, uint8_t nc, int vpd_addr, uint8_t* vpd_data, uint8_t length, char** const message );
int witchNCCheckFlashRS485 ( feenRC_t RC, uint8_t nc, uint8_t* discovery, char** const message );

/*
 * NC SPI Flash Operations
 */

int witchNCCheckFlash       ( feenRC_t RC, uint8_t nc, uint8_t* discovery, char** const message );
int witchNCIdentifyBitStream( feenRC_t RC, uint8_t nc, int offset, char* const nwpRev, char** const message );
int witchNCIdentifySlof     ( feenRC_t RC, uint8_t nc, char* const slofRev, char** const message );

#endif /* WITCHLIB_H_ */

